/* 
* Hardware.cpp
*
* Created: 11.06.2018 11:00:20
* Author: alex
*/


#include "Hardware.h"

bool Hardware::lastButton1State = false;
bool Hardware::lastButton2State = false;
bool Hardware::lastTriggerState = false;
bool Hardware::lastSwitchState = false;

void (*Hardware::photosensorCallback)(bool);
void (*Hardware::hallsensorCallback)(bool);
void (*Hardware::button1Callback)(bool);
void (*Hardware::button2Callback)(bool);
void (*Hardware::triggerCallback)(bool);
void (*Hardware::switchCallback)(bool);

uint32_t Hardware::lastButtonInterrupt = 0;
uint32_t Hardware::lastTriggerInterrupt = 0;

// default constructor
Hardware::Hardware() {
	init();
} //Hardware

void Hardware::init() {
	// setting inputs
	DDRD &= ~(1 << PHOTO) & ~(1 << HALL) & ~(1 << TRIGGER) & ~(1 << SWITCH);
	DDRB &= ~(1 << BUTTON1) & ~(1 << BUTTON2);
	
	// setting outputs
	DDRD |= (1 << BB_LED); // Blackbox LED yellow
	DDRB |= (1 << SERVO) | (1 << LED1) | (1 << LED2);	
	
	// initialize interrupts
	// trigger on any logical pin change
	EICRA = 0; //&= ~(1 << ISC11) & (1 << ISC01);
	EICRA |= (1 << ISC10) | (1 << ISC00);
	EIMSK |= (1 << INT0) | (1 << INT1); // turning photosensor- and hallsensor-interrupts on
	
	// configure PCINT
	PCICR |= (1 << PCIE0) | (1 << PCIE2); // enable PCINT-channels
	PCMSK2 |= (1 << PCINT20) | (1 << PCINT21); // set mask for trigger and switch
	PCMSK0 |= (1 << PCINT2) | (1 << PCINT3); // set mask for button1 and button2

	photosensorCallback = &Hardware::emptyCallback;
	hallsensorCallback = &Hardware::emptyCallback;
	button1Callback = &Hardware::emptyCallback;
	button2Callback = &Hardware::emptyCallback;
	triggerCallback = &Hardware::emptyCallback;
	switchCallback = &Hardware::emptyCallback;
	
	Servo::init(SERVO_CLOSED);

	sei(); // enable interrupts
}

void Hardware::attachPhotosensor(void (*f)(bool)) {
	photosensorCallback = f;
}

void Hardware::attachHallsensor(void (*f)(bool)) {
	hallsensorCallback = f;
}

void Hardware::attachButton1(void (*f)(bool)) {
	button1Callback = f;
}

void Hardware::attachButton2(void (*f)(bool)) {
	button2Callback = f;
}

void Hardware::attachTrigger(void (*f)(bool)) {
	triggerCallback = f;
}

void Hardware::attachSwitch(void (*f)(bool)) {
	switchCallback = f;
}

void Hardware::switchLED1(bool state) {
	if (state) PORTB |= (1 << LED1);
	else PORTB &= ~(1 << LED1);
}

void Hardware::switchLED2(bool state) {
	if (state) PORTB |= (1 << LED2);
	else PORTB &= ~(1 << LED2);
}

uint32_t Hardware::shot() {
	return Servo::shot(200);
}

void Hardware::emptyCallback(bool state) {
	// Hi there, we won't do anything here
}



// default destructor
Hardware::~Hardware() {

} //~Hardware

ISR(INT0_vect) {
	// photosensor triggered
	bool pin = (PIND & (1 << PHOTO));
	(*Hardware::photosensorCallback)(pin);
}

ISR(INT1_vect) {
	// hallsensor triggered
	bool pin = (PIND & (1 << HALL));
	(*Hardware::hallsensorCallback)(pin);
}

ISR(PCINT0_vect) {
	if (Time::millis() - Hardware::lastButtonInterrupt > 10) { // debounce
		bool button1 = (PINB & (1 << BUTTON1));
		bool button2 = (PINB & (1 << BUTTON2));

		if (button1 && !Hardware::lastButton1State) (*Hardware::button1Callback)(button1);
		if (button2 && !Hardware::lastButton2State) (*Hardware::button2Callback)(button2);

		Hardware::lastButton1State = button1;
		Hardware::lastButton2State = button2;
		Hardware::lastButtonInterrupt = Time::millis();
	}
}

ISR(PCINT2_vect) {
	if (Time::millis() - Hardware::lastTriggerInterrupt > 100) { // debounce
		bool trigger = (PIND & (1 << TRIGGER));
		bool switchState = (PIND & (1 << SWITCH));

		if (trigger && !Hardware::lastTriggerState) (*Hardware::triggerCallback)(trigger);
		if (switchState && !Hardware::lastSwitchState) (*Hardware::switchCallback)(switchState);

		Hardware::lastTriggerState = trigger;
		Hardware::lastSwitchState = switchState;
		Hardware::lastTriggerInterrupt = Time::millis();
	}
}
