/** \file Uart.cpp
 * Implementation of the UART
 * Abstraction Layer of the Hardware
 *
 * \date 22.01.2015 22:40:20
 * \author Alexander Detsch
 * \version 1.5
 */


#include "Uart.h"

char Uart::receivedUartString[UART_MAXSTRLEN + 1] = {};
volatile bool Uart::uartStrComplete = false;
volatile uint8_t Uart::uartStrCount = 0;
volatile uint8_t Uart::uartString[UART_MAXSTRLEN + 1] = {};

Uart::Uart() {	
} //Uart

// default destructor
Uart::~Uart() {	
} //~Uart

void Uart::begin(uint32_t baudrate, uint8_t config) {
	cli();
	
	
	// set baudrate and asynchronous double speed
	uint16_t baud = (F_CPU / 4 / baudrate - 1) / 2;
	UBRR0H = baud >> 8;
	UBRR0L = baud;
	UCSR0A |= (1 << U2X0);	
	
	/*
	// set baudrate and asynchronous normal speed
	uint16_t baud = (F_CPU / 8 / baudrate - 1) / 2;
	UBRR0H = baud >> 8;
	UBRR0L = baud;
	*/
	
	UCSR0B |= (1 << TXEN0) | (1 << RXEN0) | (1 << RXCIE0);	// Enable TX, RX, RX-Interrupt
	UCSR0C = config; // set mode of the UART such as 8N1, 8N2, ...
		
	receivedUartString[0] = 0;
		
	// Flush Receive-Buffer (entfernen evtl. vorhandener ungltiger Werte)
	do {
		UDR0;
	} while (UCSR0A & (1 << RXC0));
		
	sei();
}

void Uart::begin(uint32_t baudrate) {
	begin(baudrate, SERIAL_8N1); // use 8N1 as standard
}

/**
Send a single char directly over the UART

Wait until the transmit buffer is empty and then send the byte.

@param data byte that should be transmitted
@ingroup physTransmit
*/
void Uart::putChar(char data) {
	while(!(UCSR0A & (1 << UDRE0))); // wait until UDR empty USR = UCSRA
	UDR0 = data; // send
}

/**
Send a single byte directly over the UART

Wait until the transmit buffer is empty and then send the byte.

@param data byte that should be transmitted
@ingroup physTransmit
*/
void Uart::putByte(uint8_t data) {
	while(!(UCSR0A & (1 << UDRE0))); // wait until UDR empty USR = UCSRA
	UDR0 = data; // send
}

void Uart::empty() {
	uartStrCount = 0;
	uartStrComplete = false;
	for (uint8_t i = 0; i < UART_MAXSTRLEN; i++) {
		receivedUartString[i] = 0;
		uartString[i] = 0;
	}
}

/**
Empty the transmit buffer

Wait until the transmit buffer is empty and then returns.

@ingroup physTransmit
*/
void Uart::flush() {
	while(!(UCSR0A & (1 << UDRE0))); // wait until UDR empty USR = UCSRA
}

/**
Send a bunch of chars over the UART.

Sends all passed chars in the array one after another

@param buffer char array that should be transmitted
@ingroup transmit
*/
void Uart::printChar(char const buffer[]) {
	for(uint8_t i = 0; buffer[i] != 0; i++)
	putChar(buffer[i]);
}

/**
Send a formatted string over the UART

All format options for printf can be used.

@param string String that should be send
@param formatOptions all format options known from standard-printf
@ingroup transmit
*/
void Uart::print(char const *szFormat, ...) {
	char data[UART_MAX_SEND_LENGTH];
	va_list pArgs;
	va_start(pArgs, szFormat);
	vsnprintf(data, sizeof(data)-1, szFormat, pArgs);
	va_end(pArgs);
	printChar(data);
}

// just a f..ing bunch of wrappers. Nothing special.

/**
@name Print Numbers in different flavours.
No CR at the end of the transmission.

@param number number that should be formatted and transmitted.
@ingroup print
*/
//@{
///Print float
void Uart::print(float number) {
	print("%f", (double)number);
}

/**
@brief Print double
@param number number that should be formatted and transmitted.
*/
void Uart::print(double number) {
	print("%f", number);
}

/**
@brief Print 32-bit unsigned Integer
@param number number that should be formatted and transmitted.
*/
void Uart::print(uint32_t number) {
	print("%lu", number);
}

/**
@brief Print 16-bit unsigned Integer
@param number number that should be formatted and transmitted.
*/
void Uart::print(uint16_t number) {
	print("%u", number);
}

/**
@brief Print 8-bit unsigned Integer
@param number number that should be formatted and transmitted.
*/
void Uart::print(uint8_t number) {
	print("%u", number);
}

/**
@brief Print 32-bit signed Integer
@param number number that should be formatted and transmitted.
*/
void Uart::print(int32_t number) {
	print("%ld", number);
}

/**
@brief Print 16-bit signed Integer
@param number number that should be formatted and transmitted.
*/
void Uart::print(int16_t number) {
	print("%d", number);
}

/**
@brief Print 8-bit signed Integer
@param number number that should be formatted and transmitted.
*/
void Uart::print(int8_t number) {
	print("%i", number);
}
//@}

// now also with carriage return.

/**
Send a formatted string over the UART

All format options for printf can be used.
Function adds an CR/LF at the end of the string

@param string String that should be send
@param formatOptions all format options known from standard-printf
@ingroup transmit
*/
void Uart::println(char const buffer[]) {
	print(buffer);
	printChar("\r\n");
}

/**
@name Print Numbers in different flavours.
Add a CR at the end of the transmission.

@param number number that should be formatted and transmitted.
@ingroup println
*/
//@{
///Print float
void Uart::println(float number) {
	print("%f\r\n", (double)number);
}

/**
@brief Print double
@param number number that should be formatted and transmitted.
*/
void Uart::println(double number) {
	print("%f\r\n", number);
}

/**
@brief Print 32-bit unsigned Integer
@param number number that should be formatted and transmitted.
*/
void Uart::println(uint32_t number) {
	print("%lu\r\n", number);
}

/**
@brief Print 16-bit unsigned Integer
@param number number that should be formatted and transmitted.
*/
void Uart::println(uint16_t number) {
	print("%u\r\n", number);
}

/**
@brief Print 8-bit unsigned Integer
@param number number that should be formatted and transmitted.
*/
void Uart::println(uint8_t number) {
	print("%u\r\n", number);
}

/**
@brief Print 32-bit signed Integer
@param number number that should be formatted and transmitted.
*/
void Uart::println(int32_t number) {
	print("%ld\r\n", number);
}

/**
@brief Print 16-bit signed Integer
@param number number that should be formatted and transmitted.
*/
void Uart::println(int16_t number) {
	print("%d\r\n", number);
}

/**
@brief Print 8-bit signed Integer
@param number number that should be formatted and transmitted.
*/
void Uart::println(int8_t number) {
	print("%i\r\n", number);
}
//@}

// Now the receiving part ==========================================================================================

/**
Get the number of bytes the UART received and still hold in the receive buffer.

@return number of bytes yet received
@ingroup receiving
*/
uint8_t Uart::available() {
	return uartStrCount;
}

/**
Get information, if one complete line was received.
A line always end with a CR/LF.

@return true when the line is complete
@ingroup receiving
*/
bool Uart::allAvailable() {
	return uartStrComplete;
}

// NOT WORKING, really, please dont use this function
char Uart::getLastByte() {
	if (uartStrCount != 0) return uartString[uartStrCount - 1]; // we are in the middle of an transmission, just get the last byte
	else for (uint8_t i = 1; i < UART_MAXSTRLEN; i++) if (uartString[i] == '\0') return uartString[i - 1]; // the transmission has ended, search for the null-byte and return the previous byte (start with i=1: if the first byte is the null-byte then we received nothing)
	return -1; // we received nothing
}

/**
Returns the whole received string (until CRLF).
Wait until the line is completed

@return all received data-bytes without the CR/LF
@ingroup receiving
*/
const char* Uart::readLine() {
	while (!uartStrComplete);
	for (uint8_t i = 0; i < UART_MAXSTRLEN; i++) { // copy the buffer to a new one, so we can rewrite our own one and can work with the other one
		receivedUartString[i] = uartString[i];
	}
	uartStrCount = 0;
	uartStrComplete = false; // Weve read all, you can clear the buffer
	return receivedUartString;
}

/**
Access all yet received bytes.

@param data pointer to the byte-array where the data should be stored
@param length length of the array so no more data will be stored
@return number of bytes that have actually been stored
@ingroup receiving
*/
uint8_t Uart::read(uint8_t *data, uint8_t length) {
	uint8_t i = 0;
	for (i = 0; i < uartStrCount && i < length; i++) { // copy the buffer to a new one, so we can rewrite our own one and can work with the other one
		data[i] = uartString[i];
	}
	uartStrCount = 0; // reset the counter cause we deliverd all bytes
	uartStrComplete = false; // Weve read all, you can clear the buffer
	return i+1; // return number of received bytes;
}

/**
Access a byte at a specific position in receive-buffer

@param index index in the receiving array that shall be accessed
@return byte at the specified position
@ingroup receiving
*/
uint8_t Uart::getByteAt(uint8_t index) {
	if (index < 0 || index > uartStrCount) return 0;
	else return uartString[index];
}

void Uart::stash(uint8_t data) {
	if (uartStrCount < UART_MAXSTRLEN-1) uartString[uartStrCount++] = data;
	/*
	if (!uartStrComplete) { // received the full string yet? (CR or buffer full)
		if( data != '\n' &&	data != '\r' &&	uartStrCount < UART_MAXSTRLEN ) {
			uartString[uartStrCount++] = data;
		} else { // buffer full or CR --> all ready for shipping to customer
			uartString[uartStrCount] = '\0';
			uartStrCount = 0;
			uartStrComplete = true;
		}
	}
	*/
}

ISR(USART_RX_vect) {
	uint8_t nextChar = UDR0; // get received byte from buffer
	Uart::stash(nextChar);
}