/** \file Uart.h
 * Header for the implementation of the UART
 * Abstraction Layer of the Hardware
 *
 * \date 22.01.2015 22:40:20
 * \author Alexander Detsch
 * \version 1.5
 */


#ifndef __UART_H__
#define __UART_H__

#include <avr/interrupt.h>
#include <stdlib.h>
#include <stdint.h>
#include <stdio.h>
#include <stdarg.h>
#include <inttypes.h>
#define __STDC_FORMAT_MACROS

/*
Fr das korrekte Versenden von Float-Werten mssen im Linker noch folgende Bibliotheken hinzugefgt werden:
1. Gehe zu Project->Properties (ALT+F7) und whle unter AVR/GNU C Linker "Libraries" an. Unter Libraries (-Wl,-l) benutze den "Add"-Button zweimal und fge erst "libprintf_flt.a" und dann "libm.a" hinzu.
2. Gehe zu Miscellaneous und fge "-Wl,-u,vfprintf -lprintf_flt -lm" hinzu.
*/

// define for a specific UART-format
#define SERIAL_5N1 0x00
#define SERIAL_6N1 0x02
#define SERIAL_7N1 0x04
#define SERIAL_8N1 0x06
#define SERIAL_5N2 0x08
#define SERIAL_6N2 0x0A
#define SERIAL_7N2 0x0C
#define SERIAL_8N2 0x0E
#define SERIAL_5E1 0x20
#define SERIAL_6E1 0x22
#define SERIAL_7E1 0x24
#define SERIAL_8E1 0x26
#define SERIAL_5E2 0x28
#define SERIAL_6E2 0x2A
#define SERIAL_7E2 0x2C
#define SERIAL_8E2 0x2E
#define SERIAL_5O1 0x30
#define SERIAL_6O1 0x32
#define SERIAL_7O1 0x34
#define SERIAL_8O1 0x36
#define SERIAL_5O2 0x38
#define SERIAL_6O2 0x3A
#define SERIAL_7O2 0x3C
#define SERIAL_8O2 0x3E

class Uart {	
	#define UART_MAXSTRLEN			30 // size of the receive-buffer
	#define UART_MAX_SEND_LENGTH	100
	
	//variables
	public:
	protected:
	private:
		static char receivedUartString[UART_MAXSTRLEN + 1];
		static volatile bool uartStrComplete; // true .. completely received string
		static volatile uint8_t uartStrCount; // number of currently received bytes
		static volatile uint8_t uartString[UART_MAXSTRLEN + 1]; // working set of the receiving buffer

	//functions
	public:
		Uart();
		~Uart();
		static void begin(uint32_t baudrate);			// enable Uart and set the baudrate
		static void begin(uint32_t baudrate, uint8_t config); // same as above but additional set a specific Send-Configuration such as 8N1, 8N2, ... (see definitions above)
		
		static void print(char const *szFormat, ...);	// conversion of different data types to string (Float, Integer, ...) Format as like printf
		static void putByte(uint8_t data);				// send single raw Byte of data
				
		static void print(uint32_t number);				// send an unsigned integer (with conversion to string)
		static void print(uint16_t number);
		static void print(uint8_t number);
				
		static void print(int32_t number);				// send an signed integer (with conversion to string)
		static void print(int16_t number);
		static void print(int8_t number);
				
		static void print(float number);				// send a float (with conversion to string)
		static void print(double number);
		static void printChar(char const buffer[]);		// put char-array on uart
				
		// same. But with carriage-return afterwards
		static void println(uint32_t number);
		static void println(uint16_t number);
		static void println(uint8_t number);
				
		static void println(int32_t number);
		static void println(int16_t number);
		static void println(int8_t number);
				
		static void println(float number);
		static void println(double number);
		static void println(char const buffer[]);
				
		static bool allAvailable();						// checks if a full line was received
		static uint8_t available();						// returns the number of bytes received
		static const char* readLine();					// returns the bytes received until a CRLF. Waits until such a line is received
		static uint8_t read(uint8_t *data, uint8_t length); // writes all received bytes in the provided array
		static uint8_t getByteAt(uint8_t index);		// returns a byte at a specified position in the receiving buffer
		static void flush();							// waits until the send buffer is empty
		static void empty();							// empty the receive buffer
		static char getLastByte();						// returns the last received byte
		
		static void stash(uint8_t data);					// shall only be called by the ISR. Put a just received char in the receiving Buffer
		
	protected:
	private:
		Uart( const Uart &c );
		Uart& operator=( const Uart &c );
		static void putChar(char data); // send single character
}; //Uart

#endif //__UART_H__